# -*- coding: utf-8 -*-

__author__ = 'Daniël Franke'
__email__ = 'daniel.franke@zalando.de'
__version__ = '0.1.2'

from .metricz import MetricWriter, OAUTH2_ACCESS_TOKEN_URL, CREDENTIALS_DIR  # noqa



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import click
import socket
import requests
from clickclick import AliasedGroup, fatal_error
from environmental import Str
from metricz import MetricWriter, OAUTH2_ACCESS_TOKEN_URL, CREDENTIALS_DIR

main = AliasedGroup(context_settings=dict(help_option_names=['-h', '--help']))


class Configuration:
    token_url = Str('OAUTH2_ACCESS_TOKEN_URL', OAUTH2_ACCESS_TOKEN_URL)
    credentials_dir = Str('CREDENTIALS_DIR', CREDENTIALS_DIR)


def parse_tags(ctx, param, value):
    tags = {}
    for tag in value:
        if '=' in tag:
            tag_name, value = tag.split('=', 1)
            tags[tag_name] = value
        else:
            raise click.UsageError(
                'Invalid tag {}. Tags should be in the form of key=value'.format(tag))
    return tags


@main.command()
@click.argument('metric_name')
@click.argument('value', type=int)
@click.argument('tags', nargs=-1, callback=parse_tags)
def write(metric_name: str, value: int, tags: dict):
    config = Configuration()
    metric_writer = MetricWriter(
        config.token_url,
        config.credentials_dir,
        fail_silently=False
    )
    # default tag the hostname
    tags['hostname'] = socket.gethostname()
    try:
        metric_writer.write_metric(metric_name, value, tags)
    except requests.ConnectionError as e:
        reason = e.args[0].reason   # type: requests.packages.urllib3.exceptions.NewConnectionError
        _, pretty_reason = str(reason).split(':', 1)
        fatal_error(pretty_reason)
    except requests.HTTPError as e:
        fatal_error('HTTP {0} {1}'.format(e.response.status_code, e.response.text))



# -*- coding: utf-8 -*-

import datetime
import json

import requests
import tokens

CREDENTIALS_DIR = '/meta/credentials'
OAUTH2_ACCESS_TOKEN_URL = 'https://token.services.auth.zalando.com/oauth2/access_token?realm=/services'
TOKEN_RENEWAL_PERIOD = datetime.timedelta(hours=1)

KAIROSDB_URL = 'https://kairosdb.bus.zalan.do/api/v1/datapoints'

EPOCH = datetime.datetime.utcfromtimestamp(0)


class MetricWriter(object):
    """
    Interface class to write metrics to the Bus kairosdb.
    """
    def __init__(self,
                 url=OAUTH2_ACCESS_TOKEN_URL,
                 directory=CREDENTIALS_DIR,
                 fail_silently=True):
        tokens.configure(url=url, dir=directory)
        tokens.manage('uid', ['uid'])
        tokens.start()
        self.token_ts = None
        self.fail_silently = fail_silently
        self.requests = requests.session()
        self._renew_token()
        self.deferred_metrics = []

    def _renew_token(self):
        """
        Renews the oauth2 token if it's older than the renewal period.

        :return: None
        :rtype: None
        """
        now = datetime.datetime.utcnow()
        if not self.token_ts or (now - TOKEN_RENEWAL_PERIOD) > self.token_ts:
            token = tokens.get('uid')
            self.token_ts = now
            self.requests.headers.update({
                'Authorization': 'Bearer {}'.format(token)
            })

    def write_metric(self, metric_name, value, tags, timestamp=None):
        """
        Writes a metric to kairosdb.

        :param metric_name: The name of the metric.
        :type metric_name: str
        :param value: The value of the metric.
        :type value: int
        :param tags: Tags to add to the metric.
        :type tags: dict
        :param timestamp: The time (IN UTC!) to register the metric. (Default:
                          now)
        :type timestamp: datetime.datetime
        :return: None
        :rtype: None
        """
        payload = self._construct_payload(metric_name, value, tags, timestamp)
        response = self.requests.post(KAIROSDB_URL, data=json.dumps(payload))

        if not self.fail_silently:
            response.raise_for_status()

    def defer_metric(self, metric_name, value, tags, timestamp=None):
        """
        Defers a metric write to kairosdb, .

        :param metric_name: The name of the metric.
        :type metric_name: str
        :param value: The value of the metric.
        :type value: int
        :param tags: Tags to add to the metric.
        :type tags: dict
        :param timestamp: The time to register the metric. (Default: now)
        :type timestamp: datetime.datetime
        :return: None
        :rtype: None
        """
        payload = self._construct_payload(metric_name, value, tags, timestamp)
        self.deferred_metrics.append(payload)

    def write_deferred(self):
        """
        Writes all deferred metrics to kairosdb.

        :return: None
        :rtype: None
        """
        if self.deferred_metrics:
            response = self.requests.post(
                KAIROSDB_URL,
                data=json.dumps(self.deferred_metrics)
            )

            if 300 > response.status_code > 199:
                self.deferred_metrics = []

            if not self.fail_silently:
                response.raise_for_status()

    def _construct_payload(self, metric_name, value, tags, timestamp):
        """
        Constructs a metric payload.

        :param metric_name: The name of the metric.
        :type metric_name: str
        :param value: The value of the metric.
        :type value: int
        :param tags: Tags to add to the metric.
        :type tags: dict
        :param timestamp: The time to register the metric. (Default: now)
        :type timestamp: datetime.datetime
        :return: A payload dictionary.
        :rtype: dict
        """
        ts = timestamp if timestamp else datetime.datetime.utcnow()
        payload = {
            'name': metric_name,
            'timestamp': self._datetime_to_millis(ts),
            'value': int(value),
            'tags': tags,
        }

        return payload

    @staticmethod
    def _datetime_to_millis(dt):
        """
        Converts a datetime object to timestamp in milliseconds.

        :param dt: The datetime object.
        :type dt: datetime.datetime
        :return: The timestamp in millis.
        :rtype: int
        """
        return int((dt - EPOCH).total_seconds() * 1000)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import socket
from unittest.mock import MagicMock

import pytest
from click.testing import CliRunner
from metricz.cli import main

FAKE_ENV = {
    'OAUTH2_ACCESS_TOKEN_URL': 'oauth.example.com',
    'CREDENTIALS_DIR': '/meta/credentials',
    'METRICZ_DB_URL': 'kairosdb.example.com'
}


@pytest.fixture(autouse=True)
def metric_writer(monkeypatch):
    metric_writer_instance = MagicMock(name="MetricWriter()")
    monkeypatch.setattr('metricz.cli.MetricWriter', MagicMock(return_value=metric_writer_instance))
    return metric_writer_instance


@pytest.fixture(autouse=True)
def default_tags():
    return {'hostname': socket.gethostname()}


@pytest.fixture()
def runner():
    return CliRunner(env=FAKE_ENV)


def test_write_metric(runner, metric_writer, default_tags):
    result = runner.invoke(main, ['write', 'something', '1'],
                           catch_exceptions=False)
    assert result.exit_code == 0
    metric_writer.write_metric.assert_called_once_with('something', 1, default_tags)

    metric_writer.reset_mock()
    result = runner.invoke(main, ['write', 'other.thing', '24'],
                           catch_exceptions=False)
    assert result.exit_code == 0
    metric_writer.write_metric.assert_called_once_with('other.thing', 24, default_tags)


def test_write_metric_with_tags(runner, metric_writer, default_tags):
    result = runner.invoke(main, ['write', 'answer', '42', 'question=all.the.things'],
                           catch_exceptions=False)
    assert result.exit_code == 0
    expected_tags = {'question': 'all.the.things'}
    expected_tags.update(default_tags)
    metric_writer.write_metric.assert_called_once_with(
        'answer', 42, expected_tags)

    metric_writer.reset_mock()
    result = runner.invoke(main, ['write', 'other.thing', '1', 'some=tag', 'other=thing'],
                           catch_exceptions=False)
    assert result.exit_code == 0
    expected_tags = {'some': 'tag', 'other': 'thing'}
    expected_tags.update(default_tags)
    metric_writer.write_metric.assert_called_once_with(
        'other.thing', 1, expected_tags)

    metric_writer.reset_mock()
    result = runner.invoke(main, ['write', 'more', '7', 'some=crazy=tag'],
                           catch_exceptions=False)
    assert result.exit_code == 0
    expected_tags = {'some': 'crazy=tag'}
    expected_tags.update(default_tags)
    metric_writer.write_metric.assert_called_once_with(
        'more', 7, expected_tags)


def test_wrong_write_metric(runner):
    result = runner.invoke(main, ['write'], catch_exceptions=False)
    assert result.exit_code >= 1

    result = runner.invoke(main, ['write', 'something'],
                           catch_exceptions=False)
    assert result.exit_code >= 1


def test_invalid_tags_usage(runner):
    result = runner.invoke(main, ['write', 'other.thing', '1', 'sometag', 'other=thing'],
                           catch_exceptions=False)
    assert 'Invalid tag sometag. Tags should be in the form of key=value' in result.output
    assert result.exit_code == 2


def test_missing_configs_uses_defaults():
    env_missing_envs = {}
    return CliRunner()
    result = runner.invoke(main, ['write', 'thing', '1'],
                           env=env_missing_envs, catch_exceptions=False)
    assert result.exit_code == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'requests',
    'stups-tokens',
    'click',
    'clickclick',
    'environmental>=1.0',
]

test_requirements = [
    'pytest'
]

setup(
    name='metricz',
    version='0.1.2',
    description="Metricz makes it easy to write your metrics to the Bus kairosdb instance.",
    long_description=readme + '\n\n' + history,
    author="Daniël Franke",
    author_email='daniel.franke@zalando.de',
    url='https://github.com/bus/metricz',
    packages=[
        'metricz',
    ],
    package_dir={'metricz':
                 'metricz'},
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,
    keywords='metricz',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    setup_requires=['pytest-runner'],
    tests_require=test_requirements,
    entry_points={'console_scripts': ['metricz = metricz.cli:main']}
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_metricz
----------------------------------

Tests for `metricz` module.
"""

import pytest


from metricz import metricz


class TestMetricz(object):

    @classmethod
    def setup_class(cls):
        pass

    def test_something(self):
        pass

    @classmethod
    def teardown_class(cls):
        pass
